@@ -143,7 +143,7 @@ void show_title_bar(void)
 	wbkgd(title_bar_window, COLOR_PAIR(PT_COLOR_HEADER_BAR));   
 	werase(title_bar_window);
 
-	print(title_bar_window, 0, 0,  "     PowerTOP version 1.9       (C) 2007 Intel Corporation");
+	print(title_bar_window, 0, 0,  "     PowerTOP version 1.10      (C) 2007 Intel Corporation");
 
 	wrefresh(title_bar_window);
 
@@ -163,15 +163,18 @@ void show_title_bar(void)
 
 void show_cstates(void) 
 {
-	int i;
+	int i, count = 0;
 	werase(cstate_window);
 
-	for (i=0; i<6; i++) {
+	for (i=0; i < 10; i++) {
 		if (i == topcstate+1)
 			wattron(cstate_window, A_BOLD);
 		else
 			wattroff(cstate_window, A_BOLD);			
-		print(cstate_window, i, 0, "%s", cstate_lines[i]);
+		if (strlen(cstate_lines[i]) && count <= 6) {
+			print(cstate_window, count, 0, "%s", cstate_lines[i]);
+			count++;
+		}
 	}
 
 	for (i=0; i<5; i++) {
@@ -244,7 +247,12 @@ void show_timerstats(int nostats, int ticktime)
 					wattron(timerstat_window, A_BOLD);
 				else
 					wattroff(timerstat_window, A_BOLD);
-				print(timerstat_window, i+1, 0," %5.1f%% (%5.1f)   %s \n", lines[i].count * 100.0 / linectotal,
+				if (showpids)
+					print(timerstat_window, i+1, 0," %5.1f%% (%5.1f)   [%6s] %s \n", lines[i].count * 100.0 / linectotal,
+						lines[i].count * 1.0 / ticktime, 
+						lines[i].pid, lines[i].string);
+				else
+					print(timerstat_window, i+1, 0," %5.1f%% (%5.1f)   %s \n", lines[i].count * 100.0 / linectotal,
 						lines[i].count * 1.0 / ticktime, 
 						lines[i].string);
 				}
