@@ -35,17 +35,20 @@
 #include <assert.h>
 #include <locale.h>
 #include <time.h>
+#include <sys/stat.h>
 
 #include "powertop.h"
 
 uint64_t start_usage[8], start_duration[8];
 uint64_t last_usage[8], last_duration[8];
-
+char cnames[8][10];
 
 double ticktime = 15.0;
 
 int interrupt_0, total_interrupt;
 
+int showpids = 0;
+
 static int maxcstate = 0;
 int topcstate = 0;
 
@@ -62,7 +65,8 @@ struct irqdata {
 
 struct irqdata interrupts[IRQCOUNT];
 
-#define FREQ 3579.545
+#define FREQ_ACPI 3579.545
+static unsigned long FREQ;
 
 int nostats;
 
@@ -94,6 +98,27 @@ void push_line(char *string, int count)
 		lines = realloc (lines, (linesize ? (linesize *= 2) : (linesize = 64)) * sizeof (struct line));
 	lines[linehead].string = strdup (string);
 	lines[linehead].count = count;
+	lines[linehead].pid[0] = 0;
+	linehead++;
+}
+
+void push_line_pid(char *string, int count, char *pid) 
+{
+	int i;
+	assert(string != NULL);
+	for (i = 0; i < linehead; i++)
+		if (strcmp(string, lines[i].string) == 0) {
+			lines[i].count += count;
+			if (pid && strcmp(lines[i].pid, pid)!=0)
+				lines[i].pid[0] = 0;
+			return;
+		}
+	if (linehead == linesize)
+		lines = realloc (lines, (linesize ? (linesize *= 2) : (linesize = 64)) * sizeof (struct line));
+	lines[linehead].string = strdup (string);
+	lines[linehead].count = count;
+	if (pid)
+		strcpy(lines[linehead].pid, pid);
 	linehead++;
 }
 
@@ -236,7 +261,7 @@ static void do_proc_irq(void)
 	fclose(file);
 }
 
-static void read_data(uint64_t * usage, uint64_t * duration)
+static void read_data_acpi(uint64_t * usage, uint64_t * duration)
 {
 	DIR *dir;
 	struct dirent *entry;
@@ -286,6 +311,136 @@ static void read_data(uint64_t * usage, uint64_t * duration)
 	closedir(dir);
 }
 
+static void read_data_cpuidle(uint64_t * usage, uint64_t * duration)
+{
+	DIR *cpudir;
+	DIR *dir;
+	struct dirent *entry;
+	FILE *file = NULL;
+	char line[4096];
+	char filename[128], *f;
+	int len, clevel = 0;
+
+	memset(usage, 0, 64);
+	memset(duration, 0, 64);
+
+	cpudir = opendir("/sys/devices/system/cpu");
+	if (!cpudir)
+		return;
+
+	/* Loop over cpuN entries */
+	while ((entry = readdir(cpudir))) {
+		if (strlen(entry->d_name) < 3)
+			continue;
+
+		if (!isdigit(entry->d_name[3]))
+			continue;
+
+		len = sprintf(filename, "/sys/devices/system/cpu/%s/cpuidle",
+			      entry->d_name);
+
+		dir = opendir(filename);
+		if (!dir)
+			return;
+
+		clevel = 0;
+
+		/* For each C-state, there is a stateX directory which
+		 * contains a 'usage' and a 'time' (duration) file */
+		while ((entry = readdir(dir))) {
+			if (strlen(entry->d_name) < 3)
+				continue;
+			sprintf(filename + len, "/%s/desc", entry->d_name);
+			file = fopen(filename, "r");
+			if (file) {
+
+				memset(line, 0, 4096);
+				f = fgets(line, 4096, file);
+				fclose(file);
+				if (f == NULL)
+					break;
+			
+				f = strstr(line, "MWAIT ");
+				if (f) {
+					f += 6;
+					clevel = (strtoull(f, NULL, 16)>>4) + 1;
+				}
+				sprintf(cnames[clevel], "C%i", clevel);
+				f = strstr(line, "POLL IDLE");
+				if (f) {
+					clevel = 0;
+					sprintf(cnames[clevel], "%s", _("polling"));
+				}
+
+				f = strstr(line, "ACPI HLT");
+				if (f) {
+					clevel = 1;
+					sprintf(cnames[clevel], "%s", "C1 halt");
+				}
+			}
+			sprintf(filename + len, "/%s/usage", entry->d_name);
+			file = fopen(filename, "r");
+			if (!file)
+				continue;
+
+			memset(line, 0, 4096);
+			f = fgets(line, 4096, file);
+			fclose(file);
+			if (f == NULL)
+				break;
+
+			usage[clevel] += 1+strtoull(line, NULL, 10);
+
+			sprintf(filename + len, "/%s/time", entry->d_name);
+			file = fopen(filename, "r");
+			if (!file)
+				continue;
+		
+			memset(line, 0, 4096);
+			f = fgets(line, 4096, file);
+			fclose(file);
+			if (f == NULL)
+				break;
+
+			duration[clevel] += 1+strtoull(line, NULL, 10);
+
+			clevel++;
+			if (clevel > maxcstate)
+				maxcstate = clevel;
+		
+		}
+		closedir(dir);
+
+	}
+	closedir(cpudir);
+}
+
+static void read_data(uint64_t * usage, uint64_t * duration)
+{
+	int r;
+	struct stat s;
+
+	/* Then check for CPUidle */
+	r = stat("/sys/devices/system/cpu/cpu0/cpuidle", &s);
+	if (!r) {
+		read_data_cpuidle(usage, duration);
+		
+		/* perform residency calculations based on usecs */
+		FREQ = 1000;
+		return;
+	}
+
+	/* First, check for ACPI */
+	r = stat("/proc/acpi/processor", &s);
+	if (!r) {
+		read_data_acpi(usage, duration);
+
+		/* perform residency calculations based on ACPI timer */
+		FREQ = FREQ_ACPI;
+		return;
+	}
+}
+
 void stop_timerstats(void)
 {
 	FILE *file;
@@ -320,7 +475,9 @@ void sort_lines(void)
 	qsort (lines, linehead, sizeof (struct line), line_compare);
 }
 
-void print_battery(void)
+
+
+int print_battery_proc(void)
 {
 	DIR *dir;
 	struct dirent *dirent;
@@ -332,7 +489,7 @@ void print_battery(void)
 
 	dir = opendir("/proc/acpi/battery");
 	if (!dir)
-		return;
+		return 0;
 
 	while ((dirent = readdir(dir))) {
 		int dontcount = 0;
@@ -404,9 +561,115 @@ void print_battery(void)
 	}
 
 	show_acpi_power_line(rate, cap, prev_bat_cap - cap, time(NULL) - prev_bat_time);
+	return 1;
 }
 
-char cstate_lines[6][200];
+void print_battery_sysfs(void)
+{
+	DIR *dir;
+	struct dirent *dirent;
+	FILE *file;
+	double rate = 0;
+	double cap = 0;
+
+	char filename[256];
+	
+	if (print_battery_proc())
+		return;
+
+	dir = opendir("/sys/class/power_supply");
+	if (!dir) {
+		return;
+	}
+
+	while ((dirent = readdir(dir))) {
+		int dontcount = 0;
+		double voltage = 0.0;
+		double amperes_drawn = 0.0;
+		double watts_drawn = 0.0;
+		double watts_left = 0.0;
+		char line[1024];
+
+		if (strstr(dirent->d_name, "AC"))
+			continue;
+
+		sprintf(filename, "/sys/class/power_supply/%s/present", dirent->d_name);
+		file = fopen(filename, "r");
+		if (!file)
+			continue;
+		int s;
+		if ((s = getc(file)) != EOF) {
+			if (s == 0)
+				break;
+		}
+		fclose(file);
+
+		sprintf(filename, "/sys/class/power_supply/%s/status", dirent->d_name);
+		file = fopen(filename, "r");
+		if (!file)
+			continue;
+		memset(line, 0, 1024);
+		if (fgets(line, 1024, file) != NULL) {
+			if (!strstr(line, "Discharging"))
+				dontcount = 1;
+		}
+		fclose(file);
+
+		sprintf(filename, "/sys/class/power_supply/%s/voltage_now", dirent->d_name);
+		file = fopen(filename, "r");
+		if (!file)
+			continue;
+		memset(line, 0, 1024);
+		if (fgets(line, 1024, file) != NULL) {
+			voltage = strtoull(line, NULL, 10) / 1000000.0;
+		}
+		fclose(file);
+
+		sprintf(filename, "/sys/class/power_supply/%s/energy_now", dirent->d_name);
+		file = fopen(filename, "r");
+		if (!file)
+			continue;
+		memset(line, 0, 1024);
+		if (fgets(line, 1024, file) != NULL) {
+			watts_left = strtoull(line, NULL, 10) / 1000000.0;
+		}
+		fclose(file);
+
+		sprintf(filename, "/sys/class/power_supply/%s/current_now", dirent->d_name);
+		file = fopen(filename, "r");
+		if (!file)
+			continue;
+		memset(line, 0, 1024);
+		if (fgets(line, 1024, file) != NULL) {
+			watts_drawn = strtoull(line, NULL, 10) / 1000000.0;
+		}
+		fclose(file);
+	
+		if (!dontcount) {
+			rate += watts_drawn + voltage * amperes_drawn;
+		}
+		cap += watts_left;
+		
+
+	}
+	closedir(dir);
+	if (prev_bat_cap - cap < 0.001 && rate < 0.001)
+		last_bat_time = 0;
+	if (!last_bat_time) {
+		last_bat_time = prev_bat_time = time(NULL);
+		last_bat_cap = prev_bat_cap = cap;
+	}
+	if (time(NULL) - last_bat_time >= 400) {
+		prev_bat_cap = last_bat_cap;
+		prev_bat_time = last_bat_time;
+		last_bat_time = time(NULL);
+		last_bat_cap = cap;
+	}
+
+	show_acpi_power_line(rate, cap, prev_bat_cap - cap, time(NULL) - prev_bat_time);
+}
+
+char cstate_lines[12][200];
 
 void usage()
 {
@@ -470,13 +733,16 @@ int main(int argc, char **argv)
 	do_proc_irq();
 	do_cpufreq_stats();
 	count_usb_urbs();
+	count_usb_urbs();
 
 	memset(cur_usage, 0, sizeof(cur_usage));
 	memset(cur_duration, 0, sizeof(cur_duration));
-	printf("PowerTOP 1.9    (C) 2007 Intel Corporation \n\n");
+	printf("PowerTOP 1.10    (C) 2007, 2008 Intel Corporation \n\n");
 	if (geteuid() != 0)
 		printf(_("PowerTOP needs to be run as root to collect enough information\n"));
 	printf(_("Collecting data for %i seconds \n"), (int)ticktime);
+	printf("\n\n");
+	print_intel_cstates();
 	stop_timerstats();
 
 	while (1) {
@@ -530,7 +796,7 @@ int main(int argc, char **argv)
 		memset(&cstate_lines, 0, sizeof(cstate_lines));
 		topcstate = -4;
 		if (totalevents == 0 && maxcstate <= 1) {
-			sprintf(cstate_lines[5],_("< Detailed C-state information is only available on Mobile CPUs (laptops) >\n"));
+			sprintf(cstate_lines[5],_("< Detailed C-state information is not available.>\n"));
 		} else {
 			double sleept, percentage;;
 			c0 = sysconf(_SC_NPROCESSORS_ONLN) * ticktime * 1000 * FREQ - totalticks;
@@ -542,16 +808,19 @@ int main(int argc, char **argv)
 			sprintf(cstate_lines[1], _("C0 (cpu running)        (%4.1f%%)\n"), percentage);
 			if (percentage > 50)
 				topcstate = 0;
-			for (i = 0; i < 4; i++)
+			for (i = 0; i < 8; i++)
 				if (cur_usage[i]) {
 					sleept = (cur_duration[i] - last_duration[i]) / (cur_usage[i] - last_usage[i]
 											+ 0.1) / FREQ;
 					percentage = (cur_duration[i] -
 					      last_duration[i]) * 100 /
 					     (sysconf(_SC_NPROCESSORS_ONLN) * ticktime * 1000 * FREQ);
+
+					if (cnames[i][0]==0)
+						sprintf(cnames[i],"C%i",i+1);
 					sprintf
-					    (cstate_lines[2+i], _("C%i\t\t%5.1fms (%4.1f%%)\n"),
-					     i + 1, sleept, percentage);
+					    (cstate_lines[2+i], _("%s\t\t%5.1fms (%4.1f%%)\n"),
+					     cnames[i], sleept, percentage);
 					if (maxsleep < sleept)
 						maxsleep = sleept;
 					if (percentage > 50)
@@ -625,7 +894,7 @@ int main(int argc, char **argv)
 			if (deferrable)
 				continue;
 			sprintf(line2, "%15s : %s", process, func);
-			push_line(line2, cnt);
+			push_line_pid(line2, cnt, pid);
 		}
 		if (file)
 			pclose(file);
@@ -648,7 +917,7 @@ int main(int argc, char **argv)
 			show_wakeups(wakeups_per_second, ticktime, c0 * 100.0 / (sysconf(_SC_NPROCESSORS_ONLN) * ticktime * 1000 * FREQ) );
 		}
 		count_usb_urbs();
-		print_battery();
+		print_battery_sysfs();
 		count_lines();
 		sort_lines();
 
@@ -679,7 +948,9 @@ int main(int argc, char **argv)
 				suggestion_activate();
 				ticktime = 2;
 				displaytime = -1.0;
-			}
+			} else
+			if (keychar == 'P')
+				showpids = !showpids;
 		}
 
 		if (wakeups_per_second < 0)
@@ -769,9 +1040,11 @@ int main(int argc, char **argv)
 		suggest_WOL_off();
 		suggest_writeback_time();
 		suggest_usb_autosuspend();
+		usb_activity_hint();
 
 		if (dump) {
 			print_all_suggestions();
+			display_usb_activity();
 			exit(EXIT_SUCCESS);
 		}
 
@@ -795,5 +1068,6 @@ int main(int argc, char **argv)
 
 		
 	}
+
 	return 0;
 }
