@@ -31,6 +31,7 @@
 struct line {
 	char	*string;
 	int	count;
+	char 	pid[12];
 };
 
 typedef void (suggestion_func)(void);
@@ -58,17 +59,20 @@ void suggest_xrandr_TV_off(void);
 void suggest_WOL_off(void);
 void suggest_writeback_time(void);
 void suggest_usb_autosuspend(void);
+void usb_activity_hint(void);
 
 
 
 
-extern char cstate_lines[6][200];
+extern char cstate_lines[12][200];
 extern char cpufreqstrings[5][80];
 
 extern int topcstate;
 extern int topfreq;  
 extern int dump;
 
+extern int showpids;
+
 extern char status_bar_slots[10][40];
 extern char suggestion_key;
 extern suggestion_func *suggestion_activate; 
@@ -113,6 +117,9 @@ void push_line(char *string, int count);
 void  do_cpufreq_stats(void);
 void count_usb_urbs(void);
 
+void display_usb_activity(void);
+void activate_usb_autosuspend(void);
+void print_intel_cstates(void);
 
 
 #endif
